
from .amount import AmountRequest
from .account import AccountRequest
from .attachment import AttachmentRequest

from veem.models.base import Base

from veem.utils import deseralize

class InvoiceRequest(Base):
    def __init__(self,
                 id=None,
                 status=None,
                 timeCreated=None,
                 claimLink=None,
                 payer=None,
                 clientId=None,
                 amount=None,
                 notes=None,
                 externalInvoiceRefId=None,
                 ccEmails=[],
                 purposeOfPayment=None,
                 attachments=[],
                 exchangeRateQuoteId=None,
                 dueDate=None,
                 **kwargs):

        self.id = id
        self.status = status
        self.timeCreated = timeCreated
        self.claimLink = claimLink
        self.payer = deseralize(AccountRequest, payer).json
        self.clientId = clientId
        self.amount = deseralize(AmountRequest, amount).json
        self.notes = notes
        self.externalInvoiceRefId = externalInvoiceRefId
        self.ccEmails = ccEmails
        self.purposeOfPayment = purposeOfPayment
        self.attachments = [deseralize(AttachmentRequest,
                                       attach).json for attach in attachments]
        self.exchangeRateQuoteId = exchangeRateQuoteId
        self.dueDate = dueDate



# import from python libraries
import yaml
import logging
import argparse

from veem.models.context import VeemContext

from veem.utils import file_access_check, reverse_attrdict

logger = logging.getLogger(__name__)

class ConfigLoader(object):

    # @property
    # def parser(self):
    #     parser = argparse.ArgumentParser(description='Configuration Loader')
    #     parser.add_argument('--yaml_file', type=str,
    #                         help='Path of configuration yaml file')
    #     parser.add_argument('--configs', type=str,
    #                         help='YAML string content of secrets')
    #     return parser

    def __init__(self, yaml_file=None, configs=None, **kwargs):
        #self.args = self.parser.parse_args()
        self.configs = configs
        yaml_file = yaml_file
        if not self.configs and file_access_check(yaml_file):
            with open(yaml_file, 'r') as fp:
                self.configs = fp.read()

        try:
            self.configs = yaml.load(self.configs, Loader=yaml.FullLoader)
        except Exception as e:
            logger.error('Failed to yaml load configuration: %s' % str(e))
            self.configs = {}

        if not self.configs:
            raise AttributeError('Missing configuration')

        self.configs = reverse_attrdict(self.configs)
        self.context = VeemContext(**self.configs)

    def __getattr__(self, attr):
        """ dunder getattr method from secrets"""
        return self.configs.get(attr, None)

    @property
    def environment(self):
        return self.configs.get('environment', 'SANDBOX')

    @property
    def connectionTimeout(self):
        return self.configs.get('connectionTimeout', 1000)

    @property
    def readTimeout(self):
        return self.configs.get('readTimeout', 1000)

    @property
    def retryAttempts(self):
        return self.configs.get('retryAttempts', 1)




from veem.models.base import Base
from veem.models.address import Address
from veem.models.bank_account import BankAccount

from veem.utils import deseralize
from veem.constants import AccountType

class Contact(Base):
    def __init__(self,
                 id=None,
                 externalId=None,
                 contactAccountId=None,
                 email=None,
                 firstName=None,
                 lastName=None,
                 businessName=None,
                 countryCode=None,
                 dialCode=None,
                 phoneNumber=None,
                 batchItemId=None,
                 type=None,
                 bankAccount=None,
                 address=None,
                 **kwargs):

        self._validate_constants(AccountType, type)
        self._validate_country_code(countryCode)

        self.id = id
        self.externalId = externalId
        self.contactAccountId = contactAccountId
        self.email = email
        self.firstName = firstName
        self.lastName = lastName
        self.businessName = businessName
        self.countryCode = countryCode
        self.dialCode = dialCode
        self.phoneNumber = phoneNumber
        self.batchItemId = batchItemId
        self.type = type
        self.bankAccount = deseralize(BankAccount, bankAccount)
        self.address = deseralize(Address, address)



from veem.models.base import Base

class AccountRequest(Base):
    def __init__(self,
                 type=None,
                 email=None,
                 firstName=None,
                 lastName=None,
                 businessName=None,
                 countryCode=None,
                 phoneCountryCode=None,
                 phone=None,
                 **kwargs):

        self.type = type
        self.email = email
        self.firstName = firstName
        self.lastName = lastName
        self.businessName = businessName
        self.countryCode = countryCode
        self.phoneCountryCode = phoneCountryCode
        self.phone = phone



from veem.models.base import Base

class TokenResponse(Base):
    def __init__(self,
                 access_token=None,
                 user_name=None,
                 account_id=None,
                 expires_in=None,
                 token_type=None,
                 scope=None,
                 user_id=None,
                 refresh_token=None,
                 **kwargs):

        self.accessToken = access_token
        self.username = user_name
        self.accountId = account_id
        self.expiresIn = expires_in
        self.tokenType = token_type
        self.scope = scope
        self.userId = user_id
        self.refreshToken = refresh_token



from collections.abc import Iterable

from veem.client.responses.page import PageResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi
from veem.models.batch import Batch
from veem.models.contact import Contact

class ContactClient(Base):

    def __init__(self, config, **kwargs):
        self.config = config
        self.context = config.context
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(create=('post', ''),
                                       get=('get', '/{contactId}'),
                                       list=('get', ''),
                                       createBatch=('post', '/batch'),
                                       getBatch=('get', '/batch/{batchId}')))

    def create(self, request):
        """
            Create a contact

            @param request: a ContactRequest
            @return Contact that you just requested
            @throws VeemException If the provided ContactRequest is invalid, or
                                  if creation fails.
        """
        return self._response_handler(Contact,
                                 self.client.create(
                                            access_token=self.context.token,
                                            api_route='contacts',
                                            **(request.json))
                                     )

    def get(self, contactId):
        """
            Get a specific contact by id

            @param request: contact id
            @return Contact that you just requested
            @throws VeemException If the provided contactId is invalid, or
                                  if retriving fails.
        """
        return self._response_handler(Contact,
                                 self.client.get(
                                        uri_params=dict(contactId=contactId),
                                        access_token=self.context.token,
                                        api_route='contacts')
                                    )

    def list(self, parameters={}):
        """
            Get all contacts in pagination format

            @param request: a ContactListParameters
            @return paginated Contacts for your account
        """
        query = {}
        if parameters:
            query = getattr(parameters, 'json', {})
            query['batchItemIds'] = ','.join(query.get('batchItemIds',
                                                       [])) or None
            query = {k:v for k,v in query.items() if v is not None}
        return self._list_response_handler(PageResponse,
                                 Contact,
                                 self.client.list(
                                                request_params=query,
                                                access_token=self.context.token,
                                                api_route='contacts')
                                        )

    def createBatch(self, contactRequests):
        """
            Create a batch of contacts

            @param request: a list of ContactRequest
            @return contact batch
            @throws VeemException If the provided request is invalid, or
                                  if creating fails.
        """
        contactRequests = contactRequests if isinstance(
                                contactRequests, Iterable) else []
        contactRequests = [getattr(r, 'json', {}) for r in contactRequests]
        return self._response_handler(Batch,
                                 self.client.createBatch(
                                            access_token=self.context.token,
                                            api_route='contacts',
                                            request_body=contactRequests)
                                     )

    def getBatch(self, batchId):
        """
            Get a specific contacts batch

            @param request: Batch id
            @return contact batch
            @throws VeemException If the provided batchId is invalid, or
                                  if retriving fails.
        """
        return self._response_handler(Batch,
                                 self.client.getBatch(
                                        uri_params=dict(batchId=batchId),
                                        access_token=self.context.token,
                                        api_route='contacts')
                                    )




from veem.models.base import Base

from veem.utils import deseralize
from veem.constants import AccountType

class Account(Base):
    def __init__(self,
                 type=None,
                 email=None,
                 firstName=None,
                 lastName=None,
                 businessName=None,
                 countryCode=None,
                 phoneCountryCode=None,
                 phone=None,
                 id=None,
                 name=None,
                 isContact=False,
                 accountType=None,
                 **kwargs):

        self._validate_constants(AccountType, type)
        self._validate_constants(AccountType, accountType)
        self._validate_country_code(countryCode)

        self._type = type
        self.email = email
        self.firstName = firstName
        self.lastName = lastName
        self.businessName = businessName
        self._countryCode = countryCode
        self.phoneCountryCode = phoneCountryCode
        self.phone = phone
        self.id = id
        self.name = name
        self.isContact = isContact
        self._accountType = accountType

    @property
    def countryCode(self):
        return self._countryCode

    @countryCode.setter
    def countryCode(self, value):
        self._validate_country_code(value)
        self._countryCode = value

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value):
        self._validate_constants(AccountType, value)
        self._type = value

    @property
    def accountType(self):
        return self._accountType

    @accountType.setter
    def accountType(self, value):
        self._validate_constants(AccountType, value)
        self._accountType = value

    def _generate_request(self):
        return AccountRequest(
                     type=self.type,
                     email=self.email,
                     firstName=self.firstName,
                     lastName=self.lastName,
                     businessName=self.businessName,
                     countryCode=self.countryCode,
                     phoneCountryCode=self.phoneCountryCode,
                     phone=self.phone
                )



from veem.models.base import Base

from veem.constants import (
    Direction,
    PaymentStatus,
    PaymentSortField,
    SortOrder
)

from veem.utils import deseralize

class PaymentListParameters(Base):
    def __init__(self,
                 direction=None,
                 paymentIds=[],
                 status=[],
                 sortParameters={},
                 pageNumber=0,
                 pageSize=1,
                 **kwargs):

        self._validate_constants(Direction, direction)
        self._validate_constants(PaymentStatus, status)
        self._validate_constants((PaymentSortField, SortOrder), sortParameters)

        self.direction = direction
        self.paymentIds = paymentIds
        self.status = status
        self.sortParameters = sortParameters
        self.pageNumber = pageNumber
        self.pageSize = pageSize



from veem.client.responses.page import PageResponse
from veem.client.responses.metadata import CountryInfoResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi
from veem import __API_VERSION__ as API_VERSION

class MetadataClient(Base):

    def __init__(self, config, **kwargs):

        self.config = config
        self.context = config.context
        self.url = "veem/public/v{}/country-currency-map".format(API_VERSION)
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(getCountryCurrencyMap=('get',self.url)))

    def getCountryCurrencyMap(self):
        """
            Get all supported country and currency map
        """
        return self._list_response_handler(PageResponse,
                                           CountryInfoResponse,
                                           self.client.getCountryCurrencyMap())




from .amount import AmountResponse
from .account import AccountResponse
from .attachment import AttachmentResponse
from .exchange_rate import ExchangeRateResponse

from veem.models.base import Base
from veem.models.invoice import Invoice

from veem.utils import deseralize

class InvoiceResponse(Base):
    def __init__(self,
                 id=None,
                 status=None,
                 exchangeRate=None,
                 claimLink=None,
                 payer=None,
                 clientId=None,
                 amount=None,
                 notes=None,
                 externalInvoiceRefId=None,
                 ccEmails=[],
                 purposeOfPayment=None,
                 attachments=[],
                 exchangeRateQuoteId=None,
                 dueDate=None,
                 timeCreated=None,
                 **kwargs):

        self.id = id
        self.status = status
        self.exchangeRate = deseralize(ExchangeRateResponse, exchangeRate)
        self.claimLink = claimLink
        self.payer = deseralize(AccountResponse, payer)
        self.clientId = clientId
        self.amount = deseralize(AmountResponse, amount)
        self.notes = notes
        self.externalInvoiceRefId = externalInvoiceRefId
        self.ccEmails = ccEmails
        self.purposeOfPayment = purposeOfPayment
        self.attachments = [deseralize(AttachmentResponse,
                                       attach) for attach in attachments]
        self.exchangeRateQuoteId = exchangeRateQuoteId
        self.dueDate = dueDate
        self.timeCreated = timeCreated

    @property
    def convert(self):
        return Invoice(**self.json)




from veem.models.base import Base
from veem.models.approver import Approver

from veem.utils import deseralize
from veem.constants import ApprovalStatus

class PaymentApproval(Base):
    def __init__(self,
                 approversCompleted=None,
                 status=None,
                 approversRequired=None,
                 approvers=[],
                 **kwargs):

        self._validate_constants(ApprovalStatus, status)

        self.approversCompleted = approversCompleted
        self.status = status
        self.approversRequired = approversRequired
        self.approvers = [deseralize(Approver,
                                     apr) for apr in approvers]



from veem.client.responses.page import PageResponse
from veem.client.responses.invoice import InvoiceResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi

class InvoiceClient(Base):

    def __init__(self, config, **kwargs):
        self.config = config
        self.context = config.context
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(create=('post', ''),
                                       send=('post', '/{invoiceId}/approve'),
                                       cancel=('post', '/{invoiceId}/cancel'),
                                       get=('get', '/{invoiceId}')))

    def create(self, request):
        """
            Create an Invoice

            @param request: an InvoiceRequest
            @return Invoice that you just requested
            @throws VeemException If the provided InvoiceRequest is invalid, or
                                  if creation fails.
        """
        return self._response_handler(InvoiceResponse,
                                 self.client.create(
                                            access_token=self.context.token,
                                            api_route='invoices',
                                            **(request.json))
                                     )

    def send(self, invoiceId):
        """
            Send a specific invoice by id

            @param request: invoice id
            @return Invoice that you just requested
            @throws VeemException If the provided invoiceId is invalid, or
                                  if sending fails.
        """
        return self._response_handler(InvoiceResponse,
                                 self.client.send(
                                        uri_params=dict(invoiceId=invoiceId),
                                        access_token=self.context.token,
                                        api_route='invoices')
                                    )

    def cancel(self, invoiceId):
        """
            Cancel a specific invoice by id

            @param request: invoice id
            @return Invoice that you just requested
            @throws VeemException If the provided invoiceId is invalid, or
                                  if cancelling fails.
        """
        return self._response_handler(InvoiceResponse,
                                 self.client.cancel(
                                        uri_params=dict(invoiceId=invoiceId),
                                        access_token=self.context.token,
                                        api_route='invoices')
                                    )

    def get(self, invoiceId):
        """
            Get a specific invoice by id

            @param request: invoice id
            @return Invoice that you just requested
            @throws VeemException If the provided invoiceId is invalid, or
                                  if retriving fails.
        """
        return self._response_handler(InvoiceResponse,
                                 self.client.get(
                                        uri_params=dict(invoiceId=invoiceId),
                                        access_token=self.context.token,
                                        api_route='invoices')
                                    )



from veem.client.responses.exchange_rate import ExchangeRateResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi

class ExchangeRateClient(Base):

    def __init__(self, config, **kwargs):

        self.config = config
        self.context = config.context
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(generate=('post', '/quotes')))

    def generate(self, request):
        """
            Generate exchange rate.

            @param request: an ExchangeRateRequest with from/to amount, country
                            and currency
            @return generated Fx Quote with expiry from Veem
        """
        return self._response_handler(
                        ExchangeRateResponse,
                        self.client.generate(access_token=self.context.token,
                                        api_route='exchangerates',
                                        **request.json)
                            )



from veem.models.base import Base

class ContactRequest(Base):
    def __init__(self,
                 email=None,
                 type=None,
                 firstName=None,
                 lastName=None,
                 businessName=None,
                 isoCountryCode=None,
                 phoneDialCode=None,
                 phoneNumber=None,
                 externalBusinessId=None,
                 businessAddress=None,
                 bankAccount=None,
                 batchItemId=None,
                 **kwargs):

        self.email = email
        self.type = type
        self.firstName = firstName
        self.email = email
        self.type = type
        self.firstName = firstName
        self.lastName = lastName
        self.businessName = businessName
        self.isoCountryCode = isoCountryCode
        self.phoneDialCode = phoneDialCode
        self.phoneNumber = phoneNumber
        self.externalBusinessId = externalBusinessId
        self.businessAddress = businessAddress
        self.bankAccount = bankAccount
        self.batchItemId = batchItemId



from veem.models.base import Base

class AttachmentRequest(Base):
    def __init__(self,
                 type=None,
                 name=None,
                 referenceId=None,
                 **kwargs):

        self.type = type
        self.name = name
        self.referenceId = referenceId
